# --------------------------------------------------------------------------- #
# EVENTSINSTYLE wxPython IMPLEMENTATION
# Python Code by:
#
# Andrea Gavana, @ 03 June 2009
# Latest Revision: 04 June 2009, 11.00 GMT
#
#
# TODO/Caveats List
#
# 1. The list of widgets/events with documentation should be fetched in runtime
#    instead of being hard-coded in the script;
# 2. Format event/widget description in a less C++-ish way;
# 3. Handles windows extra styles too.
#
#
# For All Kind Of Problems, Requests Of Enhancements And Bug Reports, Please
# Write To Me At:
#
# andrea.gavana@gmail.com
# gavana@kpo.kz
#
# Or, Obviously, To The wxPython Mailing List!!!
#
#
# End Of Comments
# --------------------------------------------------------------------------- #


"""
`EventsInStyle` is a small GUI which can be used to retrieve documentation about
events and widgets styles from the online SVN trunk wxWidgets docs.


Description
===========

`EventsInStyle` is a small GUI which can be used to retrieve documentation about
events and widgets styles from the online SVN trunk wxWidgets docs.

`EventsInStyle` requires a working internet connection with no firewalls
interferences: having that, just double click on a tree item and `EventsInStyle`
will download the appropriate docs for the item (either window styles, events or
both, if present).

It then saves the downloaded data in a cPickle-based file so that the next time
you need to look at the already downloaded documentation for a particular
widget/event you won't need to waste time to download it again. Obviously, over
time the saved docs become obsolete, so you can delete the saved data and freshly
download the updated documentation.


Supported Platforms
===================

`EventsInStyle` has been tested on the following platforms:

- Windows (verified on Windows XP)


License And Version
===================

`EventsInStyle` is freeware and distributed under the wxPython license.

Latest revision: Andrea Gavana @ 04 June 2009, 11.00 GMT
Version 0.1

"""


import wx
import os
import sys

import urllib2
import re
import cPickle

import wx.aui
import wx.lib.agw.customtreectrl as CT
import wx.lib.agw.genericmessagedialog as GMD

from wx.lib.embeddedimage import PyEmbeddedImage
from wx.lib.expando import ExpandoTextCtrl
from wx.lib.stattext import GenStaticText as StaticText

from threading import Thread


_widgets = {"1Standard": ['App', 'BitmapButton', 'Button', 'CheckBox', 'CheckListBox',
                         'Choice', 'Choicebook', 'CollapsiblePane', 'ColourPickerCtrl',
                         'ComboBox', 'Control', 'DatePickerCtrl', 'Dialog', 'DirDialog',
                         'DirPickerCtrl', 'FileDialog', 'FilePickerCtrl',
                         'FontPickerCtrl', 'Frame', 'Gauge', 'GenericDirCtrl',
                         'HtmlListBox', 'HyperlinkCtrl', 'LayoutAlgorithm', 'Listbook',
                         'ListBox', 'ListCtrl', 'MenuItem', 'MessageDialog', 'MiniFrame',
                         'MultiChoiceDialog', 'Notebook', 'Panel', 'PickerBase', 'Process',
                         'ProgressDialog', 'RadioBox', 'RadioButton', 'SashWindow',
                         'ScrollBar', 'SearchCtrl', 'SimpleHtmlListBox', 'SingleChoiceDialog',
                         'Slider', 'SpinButton', 'SpinCtrl', 'SplitterWindow', 'StaticLine',
                         'StaticText', 'StatusBar', 'TaskBarIcon', 'TextCtrl', 'ToggleButton',
                         'ToolBar', 'Toolbook', 'TopLevelWindow', 'Treebook', 'TreeCtrl',
                         'Window'],
            "Animate": ['AnimationCtrl'],
            "Aui": ['AuiManager', 'AuiNotebook'],
            "Calendar": ['CalendarCtrl'],
            "Combo": ['BitmapComboBox', 'ComboCtrl', 'OwnerDrawnComboBox'],
            "Gizmos": ['EditableListBox'],
            "Html": ['HtmlWindow'],
            "Richtext": ['RichTextCtrl'],
            "Stc": ['StyledTextCtrl'],
            "Wizard": ['Wizard']
           }

_events = {"1Standard": ['ActivateEvent', 'CalculateLayoutEvent', 'ChildFocusEvent',
                        'ClipboardTextEvent', 'CloseEvent', 'CollapsiblePaneEvent',
                        'ColourPickerEvent', 'CommandEvent', 'ContextMenuEvent',
                        'DropFilesEvent', 'EraseEvent', 'FileDirPickerEvent', 'FocusEvent',
                        'FontPickerEvent', 'HelpEvent', 'HyperlinkEvent', 'IconizeEvent',
                        'IdleEvent', 'InitDialogEvent', 'JoystickEvent', 'KeyEvent',
                        'ListEvent', 'MaximizeEvent', 'MenuEvent', 'MouseCaptureChangedEvent',
                        'MouseCaptureLostEvent', 'MouseEvent', 'MoveEvent', 'NavigationKeyEvent',
                        'PaintEvent', 'PowerEvent', 'ProcessEvent', 'QueryLayoutInfoEvent',
                        'SashEvent', 'ScrollEvent', 'ScrollWinEvent', 'SetCursorEvent',
                        'ShowEvent', 'SizeEvent', 'SpinEvent', 'SplitterEvent',
                        'SysColourChangedEvent', 'TreeEvent', 'UpdateUIEvent',
                        'WindowCreateEvent'],
          "Grid": ['GridEditorCreatedEvent', 'GridEvent', 'GridRangeSelectEvent', 'GridSizeEvent'],
          "Html": ['HtmlCellEvent', 'HtmlLinkEvent'],
          "Media": ['MediaEvent'],
          "Richtext": ['RichTextEvent'],
          "Wizard": ['WizardEvent']
          }


_importList = ["wx.aui", "wx.calendar", "wx.html", "wx.media", "wx.wizard",
               "wx.combo", "wx.animate", "wx.gizmos", "wx.glcanvas", "wx.grid",
               "wx.richtext", "wx.stc"]
           
_styleHeaders = ["Style Name", "Value", "Description"]
_eventHeaders = ["Event Name", "Description"]

_baseURL = "http://docs.wxwidgets.org/trunk/classwx%s.html"

_hSpace, _vSpace = 10, 5

for mod in _importList:
    module = __import__(mod)

    
#----------------------------------------------------------------------

__version__ = "0.1"
__author__ = "Andrea Gavana @ 03 Jun 2009, andrea.gavana@gmail.com"
__appname__ = "EventsInStyle"

#----------------------------------------------------------------------
about = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABHNCSVQICAgIfAhkiAAAAztJ"
    "REFUOI2VU11oW3UcPfd/703uR3Jvkpt1/bBrM7sua+m2B20sWmaFIchQ+yD44EZBRPFJirAg"
    "rsEnmQ8+iGxWBYUOlCnFOJ1CEruxjtKta9fOJMQutU3aJGu3rPm8ufm6PiVMcA/+ns7D+R1+"
    "nHN+AIC52dk3QsHgF4VCYVTXdRP+xzAAkEokwuvRtbOKopxkWfbHq4HAZCyReHByPLMFmef3"
    "Cmzx3sqpwmMFbgeDYZTL8xQho/sP9Jxa3dh8VZMObb71obxtMjB2nq7u3Fq65uN44wXvp4Px"
    "RwWoBvjq/PlnZat1eiW00xKje+HoP4yedgmdCguHnSCd1ZCrkTm1Rt598aDxdmOPNMDQ8HBY"
    "VlrjC7FOXN+QELtfxnamBInTYREI+rsF7Gvjhnij4RNfSFeaAul0el82mx02m8X3IuvGvuvL"
    "VaTyVVwL7yKnlrF6T8WvyxlsZWugAFA0nqcMON70QCuV/Iwodhg5QQjFaGiERcceE4qVOp7u"
    "NWGgm8daSkMyXYHIA+vxEmuz8i4A3wMAc+GbbxXRZBKGR0YgKSJongUxsDi4l0dfpwCJJzjq"
    "EBBNlfGdL4HLvk0ce8EhNi6gB/r7ZlnWuGDgjHRnV8uTD+oSFdnRwIgiknkaFEXQZqGxW9Tx"
    "+U9x3N3IQLIx/tXAOT8A0HM3bsT9fwQWCE0vy2Zu1HVsyBzc1JDR6ijTHOoUAzvPwHcnj7lw"
    "Gk8dkvBa/26+3Zbz3rwZKjdTGB8fr7daxErqr3nUCFCtA7lCBZFkBT8vVbASq+C4qw1vjgiw"
    "1LZb29uPmJtFmpmZ4ZxO5/uKzdZ1338VJxws4qQbpj0SrDILopXRZS3iqC2LDkMJt+LxI6Re"
    "HwPwMQUAi4uLB2Sz9PsVf2B/QS3etdiVmSJlcmUr+kCxpFIshaqVZ+cdLVLI5Ro88Xd0re3y"
    "pV/u5DT1JQoAdF0nZ9wfvK6qxWdkuzI5MTERnJ6+OOjs7bsU+jPYkkoml/Law5fd7o+2IpHI"
    "cwLHvRMOhp3VWuWzf9Xa4/E0PfH5fLKqqvPTF3/Qz5x2f/ko0ePxMF6vt2dqauqJ/3qwBolE"
    "o1H31+cmf3t7bOyVx/H+AbSlT19ONLwLAAAAAElFTkSuQmCC")

#----------------------------------------------------------------------
delete = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABHNCSVQICAgIfAhkiAAAAgdJ"
    "REFUOI3Vk81vEkEYxp/Zme2yy7JA3SIaEhC/aJoUP649mXji4j/gwas3vfXUePTvqF5IvJmY"
    "phFtjAZN00j8iBqFauxu5EOE1gKzw4yXYiC0B48+c3kOz/Ob901mCI5QcX09KgM2rwJxUUi1"
    "oJQ8OeD8W7P2/c7y8s32KMfGSysrSruwtLEgBLkmubwKKeclSMwwdGaFQqi3ml1vr1sE8GIK"
    "UCwW6TBUusUH2m2pZFLTNGKaJty4g1TSBeccnVbjTbPpNcYvpSPj7+66UrK70Ng5Y8Ygjm3C"
    "jdlwwgYghxgKriCDxzTobaytrf1dQRuZn43G+beVzVN77ToMLYDiv0ElR8QywRgDo4xkM+kb"
    "hULhfrVaTU+t0Pa8rUuXlx7mzp6+fiaTwvtPn0EIEAQC/X4flDGEzDAMw6BxXcfUBL7v70ei"
    "8Z3Khyqell/DiswikUwhNusil8shv7iIwZDg0ZPyVjab/ToFAABIQKM6js0lYVoWKNVg2zZi"
    "0SgIIQAApaDGK5OAA6kJPzqH61DAv+g/AxAAhIAcCej193VCCMhE5CBICIQQ6HZ+6Rh7wXQs"
    "w5x4LBEKW3nHdhzHDmt22IRpzEBnFJ7vy9V7q/L5s5KIJ9xSw/fbwORvFB8rmw863c6X+s72"
    "lUw6kz9xPDFnmSaGImjVtmvvXr4qe3zQ81TP/jEq/QF/M8iAOpwAFwAAAABJRU5ErkJggg==")

#----------------------------------------------------------------------
down = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABHNCSVQICAgIfAhkiAAAAj1J"
    "REFUOI19ks9LVFEUxz/3vTczTvPDRiZNLS0qIazFLKKkhSFCLaJdf0GbClpJhBFRmwhC2rpw"
    "1a9Nq4ogCYJ+k5ZgohVaOZhKYzMy08z4ft13W0zGOPPowPfCufd7PpxzOYKaOHU9tbejtetk"
    "Q7BBU0opDxBCidJaYe370pdbdy99Xq72G7WArcnt+3pSRy+HN8V06dlIz8XDYzW3ki8UMs+A"
    "/wN0PUgoGCYYCuJIEB5I5WIEDSBQa0eruwFQChT8PVBK4UnX1+oP+BcCANezcZXr14AfQG7I"
    "bGliueVK4tS7DYBj5wjt6DxwJNnUHgkFwofQDKFQmE6BopUFAZqmBeLRRN/FkeMdmWw6M1Ke"
    "es1VPAPgyUfk4P6W3sOpE+fj0YQWCOqi5OTIrqVxpYUQgkAoEk6leq6ZJdssT/4e4AIvAXQA"
    "0niN3T8n2pM797S1dHZLYYpseZ68uYjtlXA9E6lMIZSmFhbTw+8nHtyYG6vMqq/PMvvKMqO7"
    "Mx8a400HY/HYtkxplpKzgiMrAIMQv5Zyj8fGRwceDhUL63V61X8w9Ty32p5yZyLRaJ+j5xJF"
    "O4MlC+gigLUqJqdnpk/fuTKXrq7ZAAAYH134saunYTkWifa7ej4sPRtRji+nvy6dHR4Ye1fr"
    "rwMAvH307VOqv81LNG7pNVTMysznBofOvLjv5/UDCADNyE8mm5ubiznrzb3bT2/m08j1tzoz"
    "lYUK1Ehr7Qputm1Udt4uUNkwp0ouIKuJmo+qw/MRfwCOVvmSKk0KggAAAABJRU5ErkJggg==")

#----------------------------------------------------------------------
eventsBmp = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABHNCSVQICAgIfAhkiAAAAr5J"
    "REFUOI2NkFtIUwEcxr9z5k5znl11U6d0sXnZ0bQUxVsqeClLgwp76KlSH5JuCIFBdBF6qAch"
    "CqGXegmxSCxvWSqp88LUxKVOp860pu6ibqGb13Z6kkSc9b384Q+/Hx8fB39DZmZmHpycnLRX"
    "va2qGNWPGuVyuZOmaaHNZnPgX6FpWtbXp519XVnT296hdX1u6hjX6cZN2dk5xXtxHAAoKLia"
    "X1hYdMuxTsVJpb4KitpHcCm+lOTwaKm3WJSUkBXZ3FzXDWBtp4AEAJ1OJwxWxZ/nciiAJTA2"
    "ZvhtsSywJtMCQkLi4y1WaxKApd0akAC8U9JOxBmNJvC9RKj9UD2cfykn/Mnj0pcbmwSGhvQQ"
    "SwJ8GSY6bVeBQCBQ0iK/3I1NFmCBA/sVKwD0qtCgGb4nD160J1iCE+h0OmPdDsEwMdeqa7Rs"
    "yxcdqxuxshMTVqteb17XaAxsY7OOPZtX1O6OJQHAkxapPja8m+d4EBCL+PAgKR8+n+b6yLzR"
    "0tJosy/O8wH4lJe/yADAYxiG2hIQW5fL5R69UVzWezhIQaiYMNJiMrtGxqbJ509Li09mJHml"
    "pJ/KjDpyLGbqp6F/dHhg4v69u1d2thELBNLLACLiEnN7/ANDywCklZQ8bKqpb5+xLqyyDqeL"
    "dThd7IDWsHT95p1nAChym8C+tLT4CsBQT1fthTmj/hGA1pWVdZsqLEqhVmtWzuVd7Kyta13j"
    "cIW0ze7wA7Dh4WabKQCQyfyTmciE3OkfZgx+Gxz+VF+ZmpyYqnWxvHClMiJLKlekuxMAAKzW"
    "ua6uLnWfUChPPqQMj656r/5ut68GLDtXoenp7F60zLZx9hIAIObnTb4CgUSpDFFJlpdXRRSP"
    "Rwxqe82NDW8qnI5fPcQ/BAAAhSLotkgiCU46fvpM/9e2zrlZ4+DcjOHB/7A74hELgN7++QML"
    "xRPotiKh8gAAAABJRU5ErkJggg==")

#----------------------------------------------------------------------
exit = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABHNCSVQICAgIfAhkiAAAAyFJ"
    "REFUOI09k01oXGUUhp/vO9+dezM/yUzSSZMa60+JKTSzqEkXtQi1WVSwlULRVYUsuyiC0KVV"
    "cB2KKwVF6UrEgGSRRaQLmyxKKdK0mybEBM3PaGLMZO78ZX7u/T4XUzxwOPCuzvvyPgpg7ubN"
    "seHx8Q+TuVxeG4N4HtoYtAjqxdVagwgA9f39tfWHD398/+7df81XU1Ov9508+a0EwdtRu41Y"
    "C3EMIjilEGPAGJwIWgRESPf3x69NThY+Hx7+xHRarfc61epbjTjGiGC0xoggvk9voYANQ9o7"
    "O90vRFAiKM8TyuVr2bGx780/GxsDr46Oip/P45TCaU0MBPk8L01McLS5ye7KCmiNGIN6YbG5"
    "uhrsPXnSp9uVCp2DA1ylgg1DTG8v2clJRCmUtag4JnX6NOnz51FxjCmXMYeH2JUVqFTQRBE2"
    "DLGHhyR6exm9cYNXLl3CNwZnLarTIR0EDF24QPbyZSSK0MvL8OgRAmhjLa5ex9XrnJiaIjU4"
    "yF/37tF8/hwFqFaL6uws9YUFMoUCfj5PvPQAz3Uz08ZaqNXwjGGgUKD27BnlhQVssUhtcZHm"
    "48eovT2ac3O4Ugl/9BQ6ECSXxvgGkwRUrYbEMSaZpLa9jd3fxxnDwcwMnlJ4jQbRn38QF4uY"
    "wUFkIIsc1ZFqA5O0Fl2v0ykWiUol/JERdBRBGOKiCFetEldCVGEcPTRE/PQ3lIAEPhiDTjqH"
    "ajQ4Wltjf36ezNmzZK9cIdrdJd7cxJVKkD9G6vZtJJej/et9lHJ4QQIjdC3oVgvbarEzM0Pf"
    "mTO8fOcOmXPnaC0tYXp6SF+9SjAxQXP2B9oP7qMTCYyCFAaTMsZJHKOdo7W1xfr0NCO3bpG7"
    "do3sxYtgLXZrk9oXn9Ga/xlFDJkkohUJY5z6aWDgo2POfd0pl1O+tfhAoDWpfJ7k8eN4zuLC"
    "EqrTRGeS6HQP0pfhb2T9u+Xfr6t3IT/9xqlPh/zgA9/aHtHKeYAXWzxru2UJfFTCQycE53sq"
    "RB0sFne//Hh14xsFMAL975wYfDPpB1m6xHYnpkvm/9uVtqvV4i/N5lPg6D8+CEt0sLUujQAA"
    "AABJRU5ErkJggg==")

#----------------------------------------------------------------------
question = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABHNCSVQICAgIfAhkiAAAAltJ"
    "REFUOI2Vk19IU2EYxp/jGsy8KVBGsIsVpAzK1Gh2VSJ0ZTelCXPGCHNG0IWywIv+OC/6R4YI"
    "gTpqabZlLLtQMrCrLVo6hzoImrGG1TRiOxvs7BzPct/bhbpUJuYD3933e77v4XlfDjlFXGWL"
    "4xGAixy4gU99BgvAUe67uWHbfee0lF7JUOfgpFjZ4rABxP03/ODltBRPytQ7HqRESqbOge1N"
    "9myGnf111SWNl2uOqHrGgliMpbDEi2itLcsnwEhwYqqPzBvjKDbCtdXFjc01R1WP387jFy+B"
    "GCGRSiMQ5tFWW6qMCbJOVHu0Eb9rDLBuybz27c7hAF3tnaR1+eajZOpyU/tTH8UFmTqeeUW9"
    "eSgbR3HySknXuarDTeazpaqe0S9Y4iUwIoxO/sD01xiMVYfw5uMCEoKM2VAU1+srlLGkrBPV"
    "7n2L/pGJPEa4ZKk/rhr2hPEzKiCTYauHMViNZbj3ag6MERgRFn4LGJgIor3hRD6AJgDI4whP"
    "7gz5JMOpgziwPx8rmUzWxNTlxufvCTDGwBiDpnAvTGeKcdfhkziO7ACgiPhd70W1W8sLaV3r"
    "+VJlIMwjkUqDGGHQchqvPWFkGIOmsAA3GyrQ7ZqR3nlDL6b6jdcAKxSAFRG/a0xUu7UxQda1"
    "1R1Tzn2LIiGk4foQBmMETVEBbhjK0e2akcZX4WyVazWum3i0fFLWWS6UK2dDMcSTMjRF/17e"
    "CuecRL3ZYeuwe8V4cpm6RwIUTy7TLfvm6nYcZ715yHbb7hXF5T87wts4Eqdvdj4Ex0xEeO6z"
    "Gdt2sY2701++vnGp+mtp3wAAAABJRU5ErkJggg==")

#----------------------------------------------------------------------
stylesBmp = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABHNCSVQICAgIfAhkiAAAAp5J"
    "REFUOI2Fkl1IU2EYx39nbs5jm23h0X3YqtmNZGUl1SiiMsigSy+CiigxSiGKgjIIiqIQvIlV"
    "CEUXaYEXBUEfUEkEQZJoGqsuRO3DNFLnjno22c45bxdLnQX1wAPPy/Pw+7/PB/xhH6vlix/q"
    "i+8B+YD9z/x/7WvjqsfJ3mbRuMN+FSiuX45yq4SKY8soBKyA9G9AU+iNOfFAJDsOiYHba0bG"
    "Hm4TYzcXTVS42A0UAFmZ9daZ4NUutvs3eU941ng3or3F6rWzpKAkn5/tNIRj79ti+AEXEP0L"
    "0LOHOyvrK/fj9oMxAVo3GJMQi9D1RE3Uv6AfSACTgJEJyAJoitDm7e3zuvX+MnfRKKQGwfgC"
    "timchdjutvJO1WkD+gD9rx+cCi044yuyVOmOUkYir1HWAzlpdyyFy6cJ7btAy4zg0daYmAFI"
    "Vypye3Zu9a7qixfGrz+L5547O8C2SpUrRxbxqMfD5nKN0sJJ2sv6Z1XrtjhnY+np+ZDoHFK4"
    "0a6zrnwddetb+DaUQ1PHJvL9y8lTijD0FLLDhae4jIMVAbSkACFAkpDWFuTsBdgQ/tEyPdiF"
    "on+mdyy96oJlpTjdHkwz3bY9N48DIRe6mTGD5pedToBrEXAHVpM0VtBYNUV2djaKomC3p4/x"
    "+H2NuA6LXRYiw78XIYElGAzagsGgDWDKdKCRRyAQwOfzUVtbO6sUTQiiCYEpIDZtoE6bxBIm"
    "FlmWw7IshwFiWgo1bmC1WqmpqQGguroaADVuosZNvoybRDXBmGYQ1Yy5SwSYTqV7/dg7zMkz"
    "lwBI6ibdn76jmw4ABkZ1RicNhCQhIeYDxrU04PAjK5JkQZIkEBIIE6c8VzcyNTfFeQBVHc94"
    "pRUEkLfQxUQiBUDD8xQr/bY0GMEv7ZkCqPx/e6IAAAAASUVORK5CYII=")

#----------------------------------------------------------------------
mondrian = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABHNCSVQICAgIfAhkiAAAAHFJ"
    "REFUWIXt1jsKgDAQRdF7xY25cpcWC60kioI6Fm/ahHBCMh+BRmGMnAgEWnvPpzK8dvrFCCCA"
    "coD8og4c5Lr6WB3Q3l1TBwLYPuF3YS1gn1HphgEEEABcKERrGy0E3B0HFJg7C1N/f/kTBBBA"
    "+Vi+AMkgFEvBPD17AAAAAElFTkSuQmCC")

#----------------------------------------------------------------------


def ReplaceCapitals(string):
    """
    Replaces the capital letter in a string with an underscore plus the
    corresponding lowercase character.

    **Parameters:**

    * `string`: the string to be analyzed.
    """
    

    newString = ""
    for char in string:
        if char.isupper():
            newString += "_%s"%char.lower()
        else:
            newString += char

    return newString

    
def ReadStyleDocs(frame, selectedClass):
    """
    Starts the reading process of the online wxWidgets documentation.

    **Parameters:**

    * `frame`: the main application frame;
    * `selectedClass`: the class for which we seek the docs.
    """

    dlg = wx.ProgressDialog("EventsInStyle: Loading docs",
                            "Attempting to connect to the internet...", parent=frame,
                            style=wx.PD_APP_MODAL | wx.PD_ELAPSED_TIME)
    
    dlg.Pulse()
    # Run in a separate thread
    thread = ConnectionThread(frame, selectedClass)
    while thread.isAlive():
        wx.MilliSleep(300)
        dlg.Pulse()

    dlg.Destroy()
    wx.SafeYield()


def RemoveHTMLTags(data):
    """
    Removes all the HTML tags from a string.

    **Parameters:**

    * `data`: the string to be analyzed.
    """

    p = re.compile(r'<[^<]*?>')
    return p.sub('', data)


def FormatDescription(description):
    """
    Formats a wxWidgets C++ description in a more wxPython-based way.

    **Parameters:**

    * `description`: the string description to be formatted.
    """

    description = description.replace("wx", "wx.")
    description = description.replace("EVT_COMMAND", "wxEVT_COMMAND")

    return description


def FindWindowStyles(text):
    """
    Finds the windows styles and events in the input text.

    **Parameters:**

    * `text`: the wxWidgets C++ docs for a particular widget/event, stripped
              of all HTML tags.
    """

    winStyles, winEvents = {}, {}
    inStyle = inEvent = False
    for line in text:
        if "following styles:" in line:
            inStyle = True
            continue
            
        elif "Event macros" in line:
            inEvent = True
            continue
            
        elif not line.strip():
            inStyle = inEvent = False
            continue

        if inStyle:
            start = line.index(':')
            windowStyle = line[0:start]            
            styleDescription = line[start+1:]
            winStyles[windowStyle] = styleDescription
        elif inEvent:
            start = line.index(':')
            eventName = line[0:start]
            eventDescription = line[start+1:]            
            winEvents[eventName] = eventDescription

    return winStyles, winEvents


class ConnectionThread(Thread):
    """ Worker thread class to attempt connection to the internet. """
    
    def __init__(self, notifyWindow, selectedClass):
        """
        Initialize the worker thread.

        **Parameters:**

        * `notifyWindow`: the window which will receive the notification when
                          this thread finishes the work;
        * `selectedClass`: the class for which we want the documentation.
        """
        
        Thread.__init__(self)

        self._notifyWindow = notifyWindow
        self._selectedClass = selectedClass
        
        self.setDaemon(True)
        # This starts the thread running on creation, but you could
        # also make the GUI thread responsible for calling this
        self.start()
        

    def run(self):
        """ Run the worker thread. """
        
        # This is the code executing in the new thread. Simulation of
        # a long process as a simple urllib2 call

        try:
            url = _baseURL%ReplaceCapitals(self._selectedClass)
            fid = urllib2.urlopen(url)
            
            text = fid.read()
            text = RemoveHTMLTags(text).split("\n")
            data = FindWindowStyles(text)
            
            wx.CallAfter(self._notifyWindow.LoadData, data)
        except IOError:
            # Unable to get to the internet
            wx.CallAfter(self._notifyWindow.LoadData, None)
        except:
            # Some other strange error...
            wx.CallAfter(self._notifyWindow.LoadData, None)

        return

    
class NotebookPage(wx.PyScrolledWindow):
    """ A simple notebook page which holds window styles and events information. """

    def __init__(self, parent, data, num, parentName=None):
        """
        Default class constructor.

        **Parameters:**

        * `parent`: parent window. Should not be None;
        * `data`: the formatted docs to display;
        * `num`: whether to display 2 or 3 columns of data;
        * `parentName`: if not None, one of the wxPython sub-modules
                        (wx.grid, wx.animate, etc...).
        """


        wx.PyScrolledWindow.__init__(self, parent)
        
        self.SetBackgroundColour(wx.WHITE)
        self.defaultb = wx.SystemSettings.GetColour(wx.SYS_COLOUR_LISTBOX)

        if wx.Platform in ['__WXGTK__', '__WXMSW__']:
            self.colour = wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DLIGHT)
        else:
            self.colour = wx.Colour(237, 243, 254)

        self.colWidths = []
        self.dirty = True
        
        self.SetData(data, num, parentName)
        

    def SetData(self, data, num, parentName):
        """
        Actually sets the data to be displayed.

        **Parameters:**

        * `data`: the formatted docs to display;
        * `num`: whether to display 2 or 3 columns of data;
        * `parentName`: if not None, one of the wxPython sub-modules
                        (wx.grid, wx.animate, etc...).
        """

        names = data.keys()
        names.sort()
        boldFont = wx.SystemSettings.GetFont(wx.SYS_DEFAULT_GUI_FONT)
        boldFont.SetWeight(wx.BOLD)
        italicFont = wx.Font(boldFont.GetPointSize(), wx.MODERN, wx.NORMAL, wx.NORMAL, False)
        
        xpStyle = wx.TE_READONLY|wx.NO_BORDER

        if num == 3:
            headers = _styleHeaders
        else:
            headers = _eventHeaders

        for head in headers:
            st = StaticText(self, -1, head)
            st.SetFont(boldFont)
        
        for name in names:
            description = data[name].strip()
            if num == 3 and parentName != "Standard":
                pythonValue = "wx.%s.%s"%(parentName.lower(), name[2:])
            else:
                pythonValue = name.replace("wx", "wx.")

            if num == 2:
                st1 = StaticText(self, -1, pythonValue)
                st1.SetFont(italicFont)

            else:    
                colour = wx.BLUE
                try:
                    value = "%d"%eval(pythonValue)
                except AttributeError:
                    try:
                        pythonValue2 = name.replace("wx", "wx.")
                        value = "%d"%eval(pythonValue2)
                        pythonValue = pythonValue2
                    except AttributeError:
                        value = "Unavailable"
                        colour = wx.RED

                st1 = StaticText(self, -1, pythonValue)
                st1.SetFont(italicFont)
                            
                st2 = StaticText(self, -1, value)
                st2.SetForegroundColour(colour)

            st3 = ExpandoTextCtrl(self, -1, FormatDescription(description),
                                  style=xpStyle)

        self.columns = num
        self.SetScrollbars(0, 0, 0, 0, 0, 0)
        self.Bind(wx.EVT_SIZE, self.OnSize)
        self.Bind(wx.EVT_PAINT, self.OnPaint)
        self.Bind(wx.EVT_ERASE_BACKGROUND, self.OnErase)
        self.dirty = True
        

    def OnSize(self, event):
        """
        Handles the wx.EVT_SIZE event for `NotebookPage`.

        **Parameters:**

        * `event`: a `wx.SizeEvent`.
        """

        self.DoResize()
        wx.CallAfter(self.AdjustMyScrollbars)
        

    def DoResize(self):
        """ Actually handles the resizing operation. """
        
        children = self.GetChildren()
        numCols = self.columns
        numRows = len(children)/numCols
        
        winYPos = self.CalcScrolledPosition((0, 0)).y
        headerSpace = 10

        colWidths = [0]*(numCols-1)
        size = self.GetClientSize()

        for col in xrange(numCols-1):
            colW = 0
            for indx in xrange(numRows):
                itemIndex = numCols*indx + col
                window = children[itemIndex]
                colW = max(colW, window.GetEffectiveMinSize()[0])
                colWidths[col] = colW

        for indx in xrange(numRows):
            rowH = 0
            winXPos = 0
            for col in xrange(numCols):
                itemIndex = numCols*indx + col
                window = children[itemIndex]
                rowH = max(rowH, window.GetEffectiveMinSize()[1])

            for col in xrange(numCols):
                itemIndex = numCols*indx + col
                window = children[itemIndex]
                windowWidth, windowHeight = window.GetSize()
                if col > 0:
                    xPos = sum(colWidths[0:col])+_hSpace*col
                else:
                    xPos = 0
                    
                window.SetPosition((xPos+_hSpace, _vSpace + winYPos))
                if col == numCols - 1:
                    window.SetInitialSize((size.x-sum(colWidths)-_hSpace*(col+1), -1))
                                        
            winYPos += rowH + _vSpace
            if indx == 0:
                winYPos += headerSpace

        self.colWidths = colWidths
        self.maxWindowPos = winYPos - self.CalcScrolledPosition((0, 0)).y


    def OnInternalIdle(self):
        """
        This method is normally only used internally, but sometimes an application
        may need it to implement functionality that should not be disabled by an
        application defining an `OnIdle` handler in a derived class.

        This method may be used to do delayed painting, for example, and most
        implementations call `wx.Window.UpdateWindowUI` in order to send update
        events to the window in idle time.
        """
        
        if not self.colWidths:
            return

        if not self.dirty:
            return
        
        self.AdjustMyScrollbars()


    def AdjustMyScrollbars(self):
        """ Handles the positioning of scrollbars in the `wx.PyScrolledWindow. """
        
        y_pos = self.GetScrollPos(wx.VERTICAL)
        y = self.maxWindowPos + 10 + 2
        self.SetScrollbars(0, 10, 0, y/10, 0, y_pos)
        self.dirty = False

        self.Refresh()
                

    def OnPaint(self, event):
        """
        Handles the wx.EVT_PAINT event for `NotebookPage`.

        **Parameters:**

        * `event`: a `wx.PaintEvent`.
        """
        
        dc = wx.BufferedPaintDC(self)
        self.PrepareDC(dc)
        
        dc.Clear()

        children = self.GetChildren()
        if children and not self.colWidths:
            event.Skip()
            return

        rowPos = 0
        numCols = self.columns
        numRows = len(children)/numCols
        headerSpace = 10

        size = self.GetClientSize()
        dc.SetBrush(wx.TRANSPARENT_BRUSH)
        dc.SetPen(wx.Pen(wx.Colour(200, 200, 200)))

        for indx in xrange(numRows):
            rowH = 0
            for col in xrange(numCols):
                itemIndex = numCols*indx + col
                window = children[itemIndex]
                rowH = max(rowH, window.GetEffectiveMinSize()[1])

            rowPos += rowH + _vSpace
            if indx == 0:
                rowPos += headerSpace+1
            dc.DrawLine(0, rowPos, size.x, rowPos)

        startW = _hSpace
        for indx, colW in enumerate(self.colWidths):
            startW += colW + _hSpace/2*(indx+1)
            dc.DrawLine(startW, 0, startW, rowPos)


    def OnErase(self, event):
        """
        Handles the wx.EVT_ERASE_BACKGROUND event for `NotebookPage`.

        **Parameters:**

        * `event`: a `wx.EraseEvent`.
        """

        # This is intentionally empty, to reduce flicker.        
        pass

        
            
class EventsInStyle(wx.Frame):
    """ The main `EventsInStyle` application frame. """
    
    def __init__(self, parent, id=wx.ID_ANY, title="", pos=wx.DefaultPosition,
                 size=wx.DefaultSize, style=wx.DEFAULT_FRAME_STYLE):
        """
        Default class constructor.

        **Parameters:**

        * `parent`: the window parent. This may be ``None``. If it is not ``None``, the
                    frame will always be displayed on top of the parent window on Windows;
        * `id`: the window identifier. It may take a value of -1 to indicate a default value;
        * `title`: the caption to be displayed on the frame's title bar;
        * `pos`: the window position. A value of (-1, -1) indicates a default position, chosen
                 by either the windowing system or wxPython, depending on platform;
        * `size`: the window size. A value of (-1, -1) indicates a default size, chosen by
                  either the windowing system or wxPython, depending on platform;
        * `style`: the window style.
        """

        wx.Frame.__init__(self, parent, id, title, pos, size, style)        

        self._mgr = wx.aui.AuiManager(flags=wx.aui.AUI_MGR_DEFAULT|
                                      wx.aui.AUI_MGR_ALLOW_ACTIVE_PANE)
        self._mgr.SetManagedWindow(self)

        self.leftTree = CT.CustomTreeCtrl(self, -1, style=wx.TR_DEFAULT_STYLE|
                                          wx.TR_SINGLE|wx.TR_FULL_ROW_HIGHLIGHT|
                                          wx.TR_HIDE_ROOT)
        self.centerPanel = wx.aui.AuiNotebook(self, -1, style=wx.aui.AUI_NB_TOP|
                                              wx.aui.AUI_NB_TAB_SPLIT)

        self._mgr.AddPane(self.leftTree, wx.aui.AuiPaneInfo().Name("Tree").
                          Caption("wxPython Widgets").Left().MaximizeButton(True).
                          MinSize((200, 300)))
        self._mgr.AddPane(self.centerPanel, wx.aui.AuiPaneInfo().Name("Notebook").
                          CenterPane())

        self.centerPanel.SetUniformBitmapSize((16, 16))
        self.SetIcon(mondrian.GetIcon())

        try:
            dirName = os.path.dirname(os.path.abspath(__file__))
        except:
            dirName = os.path.dirname(os.path.abspath(sys.argv[0]))

        self.dirName = dirName
        self.lastSelection = 0
        
        self.GetPickledFile()        
        self.CreateMenuBar()
        self.CreateBar()
        self.PopulateTree()
        self.BindEvents()
        
        self._mgr.Update()

        self.CenterOnScreen()
        self.Show()
        

    def GetPickledFile(self):
        """ Returns the file where downloaded data is stored (if any). """

        pickledFile = os.path.join(self.dirName, __appname__ + ".pkl")
        if not os.path.isfile(pickledFile):
            self.pickledData = {}
            return

        fid = open(pickledFile, "rb")
        try:
            self.pickledData = cPickle.load(fid)
        except:
            self.pickledData = {}
            
        fid.close()
        

    def PopulateTree(self):
        """ Populates the `CT.CustomTreeCtrl` with widget and event names. """

        imageList = wx.ImageList(16, 16)
        imageList.Add(question.GetBitmap())
        imageList.Add(down.GetBitmap())

        self.leftTree.AssignImageList(imageList)
        self.leftTree.SetIndent(10)
        self.leftTree.EnableSelectionVista()

        boldFont1 = wx.SystemSettings.GetFont(wx.SYS_DEFAULT_GUI_FONT)
        boldFont2 = wx.SystemSettings.GetFont(wx.SYS_DEFAULT_GUI_FONT)
        boldFont1.SetPointSize(9)
        boldFont1.SetWeight(wx.BOLD)
        boldFont2.SetWeight(wx.BOLD)
        
        root = self.leftTree.AddRoot("Root")
        data = [("Widgets", _widgets), ("Events", _events)]
        for text, category in data:
            item = self.leftTree.AppendItem(root, text)
            self.leftTree.SetItemFont(item, boldFont1)
            keys = category.keys()
            keys.sort()
            for key in keys:
                subitem = self.leftTree.AppendItem(item, key.replace("1", ""))
                self.leftTree.SetItemFont(subitem, boldFont2)
                for klass in category[key]:
                    if klass in self.pickledData:
                        child = self.leftTree.AppendItem(subitem, klass, image=1)
                        self.leftTree.SetPyData(child, self.pickledData[klass])
                    else:
                        child = self.leftTree.AppendItem(subitem, klass, image=0)
                        
        self.leftTree.ExpandAll()
        

    def BindEvents(self):
        """ Binds the events for the main application. """

        self.leftTree.Bind(wx.EVT_TREE_ITEM_ACTIVATED, self.OnItemActivated)
        
        self.Bind(wx.EVT_MENU, self.OnDelete, id=wx.ID_DELETE)
        self.Bind(wx.EVT_MENU, self.OnExit, id=wx.ID_EXIT)
        self.Bind(wx.EVT_MENU, self.OnAbout, id=wx.ID_ABOUT)

        self.Bind(wx.EVT_MAXIMIZE, self.OnMaximize)
        self.Bind(wx.EVT_CLOSE, self.OnExit)
        

    def CreateMenuBar(self):
        """ Creates the menu bar for `EventsInStyle`. """

        # Prepare the menu bar
        menuBar = wx.MenuBar()

        # File menu
        fileMenu = wx.Menu()
        item = wx.MenuItem(fileMenu, wx.ID_DELETE, "&Delete stored data\tCtrl+D",
                           "Deletes the data already stored using cPickle")
        item.SetBitmap(delete.GetBitmap())
        fileMenu.AppendItem(item)
        
        fileMenu.AppendSeparator()
        item = wx.MenuItem(fileMenu, wx.ID_EXIT, "&Exit\tCtrl+Q",
                           "Exit EventsInStyle application")
        item.SetBitmap(exit.GetBitmap())
        fileMenu.AppendItem(item)

        # File menu
        helpMenu = wx.Menu()
        item = wx.MenuItem(helpMenu, wx.ID_ABOUT, "&About EventsInStyle...\tCtrl+A",
                           "Shows information about EventsInStyle")
        item.SetBitmap(about.GetBitmap())
        helpMenu.AppendItem(item)

        # Add menus to the menu bar
        menuBar.Append(fileMenu, "&File")
        menuBar.Append(helpMenu, "&Help")

        self.SetMenuBar(menuBar)        


    def CreateBar(self):
        """ Creates the status bar for `EventsInStyle`. """

        self.statusBar = self.CreateStatusBar(2, wx.ST_SIZEGRIP)
        self.statusBar.SetStatusWidths([-2, -1])
        self.FillStatusBar()
        

    def FillStatusBar(self):
        """ Fills the statusbar fields with different information. """

        # Get the wxPython version information
        wxPythonVersion = wx.version()
        statusText = "wxPython %s"%wxPythonVersion

        # Ah, by the way, thank you menu bar for deleting my status bar messages...
        self.statusBar.SetStatusText(statusText, 1)
        msg = "Welcome to EventsInStyle: (c) Andrea Gavana, 03 June 2009"
        self.statusBar.SetStatusText(msg, 0)


    def OnItemActivated(self, event):
        """
        Handles the wx.EVT_TREE_ITEM_ACTIVATED event for the tree control.

        **Parameters:**

        * `event`: a `wx.TreeEvent`.
        """

        item = event.GetItem()
        event.Skip()

        if self.leftTree.GetChildrenCount(item) > 0:
            return
        
        self.currentItem = item

        data = self.leftTree.GetPyData(item)
        if data is not None:
            wx.BeginBusyCursor()
            self.LoadData(data)
            wx.EndBusyCursor()
        else:
            klass = self.leftTree.GetItemText(item)
            ReadStyleDocs(self, klass)


    def LoadData(self, data):
        """
        Layouts the interface once new data are downloaded.

        **Parameters:**

        * `data`: the data to be displayed.
        """

        if data is None:
            self.RunError(2, "Unable to connect to the internet.")
            return

        self.Bind(wx.aui.EVT_AUINOTEBOOK_PAGE_CHANGED, None, self.centerPanel)
        self.Freeze()

        for indx in xrange(self.centerPanel.GetPageCount()-1, -1, -1):
            self.centerPanel.DeletePage(indx)

        styles, events = data
        
        self.leftTree.SetPyData(self.currentItem, (styles, events))
        self.leftTree.SetItemImage(self.currentItem, 1)

        img1, img2 = stylesBmp.GetBitmap(), eventsBmp.GetBitmap()

        parentName = self.leftTree.GetItemParent(self.currentItem).GetText()
        self.centerPanel.AddPage(NotebookPage(self, styles, 3, parentName), "Styles",
                                 True, img1)
        self.centerPanel.AddPage(NotebookPage(self, events, 2), "Events", False, img2)

        self.centerPanel.SetSelection(self.lastSelection)
        self.Thaw()

        wx.CallAfter(self.SendEventSize)
        self.Bind(wx.aui.EVT_AUINOTEBOOK_PAGE_CHANGED, self.OnPageChanged, self.centerPanel)


    def SendEventSize(self):
        """ Sends a `wx.SizeEvent` to the notebook pages holding the data. """
        
        sizeEvent = wx.SizeEvent(winid=self.GetId())
        for pageIndex in xrange(self.centerPanel.GetPageCount()):
            page = self.centerPanel.GetPage(pageIndex)
            page.GetEventHandler().ProcessEvent(sizeEvent)


    def RunError(self, kind, msg):
        """
        An utility method that shows a message dialog with different functionalities
        depending on the input.

        **Parameters:**
        
        * `kind`: the kind of message (error, warning, question, message);
        * `msg`: the actual message to display.
        """

        kindDict = {0: "Message", 1: "Warning", 2: "Error", 3: "Question"}
        kind = kindDict[kind]
        
        if kind == "Message":    # is a simple message
            style = wx.OK | wx.ICON_INFORMATION
        elif kind == "Warning":  # is a warning
            style = wx.OK | wx.ICON_EXCLAMATION
        elif kind == "Question": # is a question
            style = wx.YES_NO | wx.CANCEL | wx.ICON_QUESTION
        else:                    # is an error
            style = wx.OK | wx.ICON_ERROR

        # Create the message dialog
        dlg = GMD.GenericMessageDialog(self, msg, "EventsInStyle %s"%kind, style)
        dlg.SetIcon(self.GetIcon())
        answer = dlg.ShowModal()
        dlg.Destroy()

        if kind == "Question":
            # return the answer, it was a question
            return answer
        

    def OnMaximize(self, event):
        """
        Handles the wx.EVT_MAXIMIZE event for the main frame.

        **Parameters:**

        * `event`: a `wx.MaximizeEvent`.
        """
        
        wx.CallAfter(self.SendEventSize)


    def OnPageChanged(self, event):
        """
        Handles the wx.aui.EVT_AUINOTEBOOK_PAGE_CHANGED event for the notenbook.

        **Parameters:**

        * `event`: a `wx.aui.AuiNotebookEvent`.
        """
        
        event.Skip()
        
        wx.CallAfter(self.SendEventSize)
        self.lastSelection = event.GetSelection()
        
        
    def OnDelete(self, event):
        """
        Handles the ``wx.ID_DELETE`` menu item event.

        **Parameters:**

        * `event`: a `wx.MenuEvent`.
        """
        
        msg = "Are you sure you wish to delete all the downloaded data?"
        answer = self.RunError(3, msg)

        if answer != wx.ID_YES:
            return

        busy = wx.BusyInfo("Deleting downloaded data...")
        wx.SafeYield()
        
        pickledFile = os.path.join(self.dirName, __appname__ + ".pkl")
        if os.path.isfile(pickledFile):
            os.remove(pickledFile)

        self.RecurseOnChildren(item=None, pickledData=None, setData=True)
        del busy
        

    def OnExit(self, event):
        """
        Handles the ``wx.ID_EXIT`` menu item event and the `wx.CloseEvent`.

        **Parameters:**

        * `event`: a `wx.MenuEvent` or a `wx.CloseEvent`.
        """

        busy = wx.BusyInfo("Saving downloaded data...")
        wx.SafeYield()
        
        pickledFile = os.path.join(self.dirName, __appname__ + ".pkl")
        pickledData = self.RecurseOnChildren(item=None, pickledData=None)

        fid = open(pickledFile, "wb")
        cPickle.dump(pickledData, fid, cPickle.HIGHEST_PROTOCOL)
        fid.close()

        del busy
        wx.CallAfter(self.Destroy)
        

    def RecurseOnChildren(self, item, pickledData, setData=False):
        """
        Recurse on all items of the tree control, setting data on them
        or deleting data from them, depending on the value of the
        `setData` parameter.

        **Parameters:**

        * `item`: the tree item: if ``None``, the root item is used;
        * `pickledData`: a dictionary which holds the stored data;
        * `setData`: whether to set the data or to retrieve them.
        """
        
        if item is None:
            item = self.leftTree.GetRootItem()
        if pickledData is None:
            pickledData = {}

        child, cookie = self.leftTree.GetFirstChild(item)
        while child:
            if self.leftTree.GetChildrenCount(child) > 0:
                pickledData = self.RecurseOnChildren(child, pickledData, setData)
            else:
                if setData:
                    self.leftTree.SetPyData(child, None)
                    self.leftTree.SetItemImage(child, 0)
                else:
                    text = self.leftTree.GetItemText(child)
                    pydata = self.leftTree.GetPyData(child)
                    if pydata:
                        pickledData[text] = pydata

            child, cookie = self.leftTree.GetNextChild(item, cookie)

        return pickledData            


    def OnAbout(self, event):
        """ Shows the about dialog for `EventsInStyle`. """

        msg = ("This is the about dialog of EventsInStyle.\n\n" + \
              "Version %s \n" + \
              "Author: Andrea Gavana @ 03 Jun 2009\n\n" + \
              "Please report any bug/request of improvements\n" + \
              "to me at the following addresses:\n\n" + \
              "andrea.gavana@gmail.com\ngavana@kpo.kz")%__version__
              
        self.RunError(0, msg)
    

# Run the main application
if __name__ == "__main__":

    app = wx.PySimpleApp()
    frame = EventsInStyle(None, -1, "Events In Style :-D", size=(750, 600))
    app.MainLoop()

    
        